@@ -3,20 +3,23 @@
  */
 package org.gcube.portlets.user.tdtemplate.client;
 
+import java.util.ArrayList;
 import java.util.List;
 
 import org.gcube.portlets.user.tdtemplate.client.resources.TdTemplateAbstractResources;
-import org.gcube.portlets.user.tdtemplate.client.template.flow.WindowFlowCreate;
-import org.gcube.portlets.user.tdtemplate.client.template.view.TemplatePanel;
-import org.gcube.portlets.user.tdtemplate.client.template.view.TemplateSwitcherInteface;
-import org.gcube.portlets.user.tdtemplate.client.template.view.updater.TemplateUpdaterFormSwitcherPanel;
-import org.gcube.portlets.user.tdtemplate.client.template.view.updater.TemplateUpdaterGenerator;
+import org.gcube.portlets.user.tdtemplate.client.templateactions.TemplatePanelActionEdit;
+import org.gcube.portlets.user.tdtemplate.client.templatecreator.flow.WindowFlowCreate;
+import org.gcube.portlets.user.tdtemplate.client.templatecreator.view.TemplateSwitcherInteface;
+import org.gcube.portlets.user.tdtemplate.client.templatecreator.view.updater.TemplateUpdaterFormSwitcherPanel;
+import org.gcube.portlets.user.tdtemplate.client.templatecreator.view.updater.TemplateUpdaterGenerator;
 import org.gcube.portlets.user.tdtemplate.shared.ClientReportTemplateSaved;
 import org.gcube.portlets.user.tdtemplate.shared.TdColumnDefinition;
 import org.gcube.portlets.user.tdtemplate.shared.TdFlowModel;
 import org.gcube.portlets.user.tdtemplate.shared.TdTTemplateType;
 import org.gcube.portlets.user.tdtemplate.shared.TdTemplateDefinition;
 import org.gcube.portlets.user.tdtemplate.shared.TdTemplateUpdater;
+import org.gcube.portlets.user.tdtemplateoperation.shared.action.TabularDataAction;
+import org.gcube.portlets.user.tdtemplateoperation.shared.action.TabularDataActionDescription;
 
 import com.extjs.gxt.ui.client.Style.Scroll;
 import com.extjs.gxt.ui.client.widget.MessageBox;
@@ -42,7 +45,7 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 	private TdTemplateControllerUpdater INSTANCE = this;
 	
 	private TdTemplateUpdater templateUpdater;
-	private TdTemplatePrivateEventsBinder binder;
+//	private TdTemplatePrivateEventsBinder binder;
 	
 	private TemplateUpdaterGenerator tug;
 	
@@ -57,7 +60,7 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 	
 	public TdTemplateControllerUpdater(long templateId){
 		this.templateId = templateId;
-		initToolbarSubmit();
+//		initToolbarSubmit(CREATION_STATE.TEMPLATE_UPDATE);
 		binder = new TdTemplatePrivateEventsBinder();
 		binder.bindEvents(INSTANCE);
 		bindCommonEvents();
@@ -151,15 +154,18 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 	}
 	
 	@Override
-	public void createTemplate(List<TdColumnDefinition> columns){
+	public void createTemplate(List<TdColumnDefinition> columns, final boolean save){
 
 		if(columns.size()>0){
 		
 			window.mask("Updating Template Columns");
 			
 			TdFlowModel flowAttached = tug.getTemplatePanel().getFlow(); //??? TODO
+//			ACTION?? //??? TODO
 			
-			tdTemplateServiceAsync.updateTemplate(columns, new AsyncCallback<ClientReportTemplateSaved>() {
+			List<TabularDataAction> actions = actionsController.getActions();
+			
+			tdTemplateServiceAsync.updateTemplate(columns, save, actions, new AsyncCallback<ClientReportTemplateSaved>() {
 
 				@Override
 				public void onFailure(Throwable caught) {
@@ -170,9 +176,10 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 				@Override
 				public void onSuccess(ClientReportTemplateSaved result) {
 				    messageBoxTitle = TdTemplateConstants.TEMPLATE_UPDATED;
-				    messageBoxMsg = TdTemplateConstants.TEMPLATE_UPDATED_SUCCESSFULLY+"!";
 					window.unmask();
-					showReportTemplateSaved(result);
+					
+					String messageBoxOkMsg =save?"Template saved successfully":"Template updated successfully";
+					showReportTemplateSaved(result, save, messageBoxOkMsg);
 				}
 			});
 		}
@@ -183,6 +190,10 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 	 */
 	@Override
 	public void doInitTemplate(TemplateSwitcherInteface switcherInterface) {
+		this.templatePanelActionUpdater = new TemplatePanelActionEdit(switcherUpdater, this);
+		this.actionsController = new TdTemplateControllerActions(this, templatePanelActionUpdater);
+		this.templatePanelActionUpdater.setActionControlleEventBus(actionsController.getEventBus());
+		
 		switcher.mask("Updating template", TdTemplateConstants.LOADINGSTYLE);
 		tug = new TemplateUpdaterGenerator(switcherUpdater, INSTANCE);
 		tug.initTemplatePanel();
@@ -195,7 +206,7 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 
 			@Override
 			public void onFailure(Throwable caught) {
-
+				displayError("Error", "Sorry an error occurred on retrieving Flow from service");
 			}
 
 			@Override
@@ -209,11 +220,6 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 			}
 		});
 		
-	
-		
-//		switcher.hide();
-//		switcherUpdater.hide();
-		
 		int index = mainPanel.indexOf(switcherUpdater);
 		
 		if(index!=-1)
@@ -222,8 +228,8 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 
 		mainPanel.mask("Building template");
 		mainPanel.setLayout(new FitLayout());
-		TemplatePanel templatePanel = tug.getTemplatePanel();
-		mainPanel.add(templatePanel.getPanel());
+		
+		activeCard(CREATION_STATE.TEMPLATE_CREATION);
 		window.setHeading(TdTemplateConstants.TEMPLATEUPDATER+": " +switcher.getName() +" - Type: "+switcherInterface.getType());
 //		window.layout(true);
 		mainPanel.unmask();
@@ -255,7 +261,54 @@ public class TdTemplateControllerUpdater extends TdTemplateController {
 	 */
 	@Override
 	public void doUpdateTemplate() {
+		
 		tug.getUpdater().setColumns(templateUpdater.getListColumns());
+		
+		//ARE THERE ACTIONS?
+		List<TabularDataActionDescription> actionDescriptions = templateUpdater.getTabularDataActionDescription();
+		
+		if(actionDescriptions!=null && actionDescriptions.size()>0){
+			String alertMessage = "The template \""+switcherUpdater.getName() +"\" has one or more columns created by";
+			
+			if(actionDescriptions.size()>1)
+				alertMessage+= " post-action.";
+			else
+				alertMessage+= " post-actions.";
+			
+			alertMessage+= " Either you add new actions or press \"back\" to modify the template base columns";
+			
+			showTemplateViewToActions();
+			
+			//CONVETING ACTIONS
+			ArrayList<TabularDataAction> listActions = new ArrayList<TabularDataAction>();
+			for (final TabularDataActionDescription tabularDataActionDescription : actionDescriptions) {
+				TabularDataAction action = new TabularDataAction() {
+					
+					@Override
+					public String getId() {
+						return tabularDataActionDescription.getId();
+					}
+					
+					@Override
+					public String getDescription() {
+						return tabularDataActionDescription.getDescription();
+					}
+				};
+				listActions.add(action);
+			}
+			
+			actionsController.setActions(listActions);
+			String msg = TdTemplateConstants.LATEST_OPERATION+ ": performed operation "+listActions.get(listActions.size()-1).getDescription()+"";
+			actionsController.refreshLastOperationLabel(msg, "");
+			templatePanelActionUpdater.enableUndoLastOperation(true); //ENABLING UNDO
+			cardTemplate.setIsUpdate(true);
+//			MessageBox.info("Template Actions notification!", alertMessage, null);
+			
+			DialogResult dialog = getDialog(null, "Template Actions notification!", alertMessage);
+//			DialogResult dialog = new DialogResult(null, "Template Actions notification!", alertMessage);
+			dialog.getElement().getStyle().setZIndex(getWindowZIndex()+50);
+			dialog.show();
+		}
 //		tug.getTemplatePanel().getPanel().layout(true);
 		mainPanel.layout(true);
 	}
